@@ -25,6 +25,7 @@ use utils;
 use std::borrow::Cow;
 use std::borrow::Cow::{Borrowed};
 use std::collections::{HashMap, HashSet};
+use std::collections::hash_map::Entry;
 use std::cmp;
 
 #[derive(PartialEq, Debug)]
@@ -91,7 +92,7 @@ pub enum Tag<'a> {
     FootnoteDefinition(Cow<'a, str>),
 
     // tables
-    Table(i32),
+    Table(Vec<Alignment>),
     TableHead,
     TableRow,
     TableCell,
@@ -104,7 +105,7 @@ pub enum Tag<'a> {
     Image(Cow<'a, str>, Cow<'a, str>),
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub enum Event<'a> {
     Start(Tag<'a>),
     End(Tag<'a>),
@@ -116,8 +117,16 @@ pub enum Event<'a> {
     HardBreak,
 }
 
+#[derive(Copy, Clone, Debug)]
+pub enum Alignment {
+    None,
+    Left,
+    Center,
+    Right,
+}
+
 bitflags! {
-    flags Options: u32 {
+    pub flags Options: u32 {
         const OPTION_FIRST_PASS = 1 << 0,
         const OPTION_ENABLE_TABLES = 1 << 1,
         const OPTION_ENABLE_FOOTNOTES = 1 << 2,
@@ -247,7 +256,7 @@ impl<'a> RawParser<'a> {
     // Return: bytes scanned, whether containers are complete, and remaining space
     fn scan_containers(&self, text: &str) -> (usize, bool, usize) {
         let (mut i, mut space) = scan_leading_space(text, 0);
-        for container in self.containers.iter() {
+        for container in &self.containers {
             match *container {
                 Container::BlockQuote => {
                     if space <= 3 {
@@ -263,7 +272,7 @@ impl<'a> RawParser<'a> {
                         return (i, false, space);
                     }
                 }
-                Container::FootnoteDefinition => (),
+                Container::FootnoteDefinition |
                 Container::List(_, _) => (),
                 Container::ListItem(indent) => {
                     if space >= indent {
@@ -358,7 +367,7 @@ impl<'a> RawParser<'a> {
                             _ => true,
                         }
                     }).collect();
-                    if close_tags.len() != 0 {
+                    if !close_tags.is_empty() {
                         for tag in &mut close_tags {
                             tag.1 = self.off; // limit
                             tag.2 = self.off; // next
@@ -538,7 +547,7 @@ impl<'a> RawParser<'a> {
         }
         self.state = State::TableRow;
         self.off = off;
-        return self.start(Tag::TableRow, self.text.len(), 0);
+        self.start(Tag::TableRow, self.text.len(), 0)
     }
 
     fn start_hrule(&mut self) -> Event<'a> {
@@ -619,7 +628,7 @@ impl<'a> RawParser<'a> {
         let beg_info = self.off + n;
         let next_line = beg_info + scan_nextline(&self.text[beg_info..]);
         self.off = next_line;
-        let info = unescape(&self.text[beg_info..next_line].trim());
+        let info = unescape(self.text[beg_info..next_line].trim());
         let size = self.text.len();
         self.state = State::CodeLineStart;
         self.start(Tag::CodeBlock(info), size, 0)
@@ -715,14 +724,8 @@ impl<'a> RawParser<'a> {
             space < 4
         } else if space <= 3 {
             let (n, c) = scan_code_fence(tail);
-            if c != self.fence_char || n < self.fence_count {
-                return false;
-            }
-            if n < tail.len() && scan_blank_line(&tail[n..]) == 0 {
-                // Closing code fences cannot have info strings
-                return false;
-            }
-            return true;
+            c == self.fence_char && n >= self.fence_count &&
+                (n >= tail.len() || scan_blank_line(&tail[n..]) != 0)
         } else {
             false
         }
@@ -746,10 +749,26 @@ impl<'a> RawParser<'a> {
                 data.starts_with("<!")
     }
 
+    fn get_html_tag(&self) -> Option<&'static str> {
+        static TAGS: &'static [&'static str; 3] = &["</script>", "</pre>", "</style>"];
+        for tag in TAGS {
+            if self.text[self.off + 1..].starts_with(&tag[2..]) {
+                return Some(tag);
+            }
+        }
+        None
+    }
+
     fn do_html_block(&mut self) -> Event<'a> {
+        let mut i = self.off;
+        if let Some(tag) = self.get_html_tag() {
+            let text = self.text[i..].split(tag).take(1).next().unwrap_or("");
+            self.off = i + text.len();
+            self.state = State::StartBlock;
+            return Event::Html(utils::cow_append(Borrowed(""), Borrowed(&self.text[i..self.off])));
+        }
         let size = self.text.len();
         let mut out = Borrowed("");
-        let mut i = self.off;
         let mut mark = i;
         loop {
             let n = scan_nextline(&self.text[i..]);
@@ -815,9 +834,9 @@ impl<'a> RawParser<'a> {
         if linktext.is_empty() {
             return false;
         }
-        if !self.links.contains_key(&linktext) {
+        if let Entry::Vacant(entry) = self.links.entry(linktext) {
             let dest = unescape(raw_dest);
-            self.links.insert(linktext, (dest, title));
+            entry.insert((dest, title));
         }
         self.state = State::StartBlock;
         self.off += i;
@@ -963,7 +982,7 @@ impl<'a> RawParser<'a> {
             b'\t' => Some(self.char_tab()),
             b'\\' => self.char_backslash(),
             b'&' => self.char_entity(),
-            b'_' => self.char_emphasis(),
+            b'_' |
             b'*' => self.char_emphasis(),
             b'[' if self.opts.contains(OPTION_ENABLE_FOOTNOTES) => self.char_link_footnote(),
             b'[' | b'!' => self.char_link(),
@@ -975,7 +994,7 @@ impl<'a> RawParser<'a> {
 
     fn char_null(&mut self) -> Event<'a> {
         self.off += 1;
-        Event::Text(Borrowed(&"\u{fffd}"))
+        Event::Text(Borrowed("\u{fffd}"))
     }
 
     // expand tab in content (used for code and inline)
@@ -1031,7 +1050,11 @@ impl<'a> RawParser<'a> {
         while i < limit {
             let c2 = data.as_bytes()[i];
             if c2 == b'\n' && !is_escaped(data, i) {
-                let space = 0;  // TODO: scan containers
+                let (_, complete, space) = self.scan_containers(&self.text[i..]);
+                if !complete {
+                    i += 1;
+                    continue;
+                }
                 if self.is_inline_block_end(&self.text[i + 1 .. limit], space) {
                     return None
                 } else {
@@ -1115,18 +1138,19 @@ impl<'a> RawParser<'a> {
                     let n = self.scan_whitespace_inline(&data[i..]);
                     if n == 0 { return (0, 0, 0, 0); }
                     i += n;
-                    continue;
                 }
                 b'[' => {
                     nest += 1;
                     if nest == MAX_LINK_NEST { return (0, 0, 0, 0); }
-                    max_nest = cmp::max(max_nest, nest)
+                    max_nest = cmp::max(max_nest, nest);
+                    i += 1;
                 }
                 b']' => {
                     nest -= 1;
                     if nest == 0 {
                         break;
                     }
+                    i += 1;
                 }
                 b'\\' => i += 1,
                 b'<' => {
@@ -1145,9 +1169,8 @@ impl<'a> RawParser<'a> {
                         i += beg;
                     }
                 }
-                _ => ()
+                _ => i += 1
             }
-            i += 1;
         }
         let text_end = i;
         i += 1;  // skip closing ]
@@ -1209,7 +1232,7 @@ impl<'a> RawParser<'a> {
         let mut i = i + n;
 
         // scan dest
-        let (dest, title, beg, end, next) = if data[i..].starts_with("(") {
+        let (dest, title, beg, end, next) = if data[i..].starts_with('(') {
             i += 1;
             i += self.scan_whitespace_inline(&data[i..]);
             if i >= size { return None; }
@@ -1340,7 +1363,7 @@ impl<'a> RawParser<'a> {
         assert!(self.opts.contains(OPTION_ENABLE_FOOTNOTES));
         let (n_footnote, text_beg, text_end) = self.scan_footnote_label(data);
         if n_footnote == 0 { return None; }
-        return Some((&data[text_beg..text_end], n_footnote));
+        Some((&data[text_beg..text_end], n_footnote))
     }
 
     fn scan_footnote_label(&self, data: &str) -> (usize, usize, usize) {
