@@ -1,16 +1,22 @@
 // Copyright 2015 Google Inc. All rights reserved.
 //
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
 
 //! Scanners for fragments of CommonMark syntax
 
@@ -19,6 +25,7 @@ use utils;
 use std::borrow::Cow;
 use std::borrow::Cow::{Borrowed, Owned};
 use std::char;
+use parse::Alignment;
 
 pub use puncttable::{is_ascii_punctuation, is_punctuation};
 
@@ -284,6 +291,65 @@ pub fn scan_setext_header(data: &str) -> (usize, i32) {
     (i, level)
 }
 
+// returns number of bytes in line (including trailing
+// newline) and column alignments
+pub fn scan_table_head(data: &str) -> (usize, Vec<Alignment>) {
+    let (mut i, spaces) = calc_indent(data, 4);
+    if spaces > 3 || i == data.len() {
+        return (0, vec![]);
+    }
+    let mut cols = vec![];
+    let mut active_col = Alignment::None;
+    let mut start_col = true;
+    if data.as_bytes()[i] == b'|' {
+        i += 1;
+    }
+    for c in data.as_bytes()[i..].iter() {
+        let eol = scan_eol(&data[i..]);
+        if eol.1 {
+            i += eol.0;
+            break;
+        }
+        match *c {
+            b' ' => (),
+            b':' => {
+                active_col =
+                    match (start_col, active_col) {
+                        (true, Alignment::None) => Alignment::Left,
+                        (false, Alignment::Left) => Alignment::Center,
+                        (false, Alignment::None) => Alignment::Right,
+                        _ => active_col,
+                    };
+                start_col = false;
+            }
+            b'-' => {
+                start_col = false;
+            },
+            b'|' => {
+                start_col = true;
+                cols.push(active_col);
+                active_col = Alignment::None;
+            },
+            _ => {
+                cols = vec![];
+                start_col = true;
+                break;
+            },
+        }
+        i += 1;
+    }
+
+    if !start_col {
+        cols.push(active_col);
+    }
+
+    if cols.len() < 2 {
+        (0, vec![])
+    } else {
+        (i, cols)
+    }
+}
+
 // returns: number of bytes scanned, char
 pub fn scan_code_fence(data: &str) -> (usize, u8) {
     let c = data.as_bytes()[0];
@@ -409,11 +475,11 @@ pub fn scan_entity(data: &str) -> (usize, Option<Cow<'static, str>>) {
             return (end + 1, Some(Borrowed(value)));
         }
     }
-    return (0, None);
+    (0, None)
 }
 
 // note: dest returned is raw, still needs to be unescaped
-pub fn scan_link_dest<'a>(data: &'a str) -> Option<(usize, &'a str)> {
+pub fn scan_link_dest(data: &str) -> Option<(usize, &str)> {
     let size = data.len();
     let mut i = 0;
     let pointy_n = scan_ch(data, b'<');
@@ -457,7 +523,7 @@ pub fn scan_link_dest<'a>(data: &'a str) -> Option<(usize, &'a str)> {
 }
 
 // return value is: total bytes, link uri
-pub fn scan_autolink<'a>(data: &'a str) -> Option<(usize, Cow<'a, str>)> {
+pub fn scan_autolink(data: &str) -> Option<(usize, Cow<str>)> {
     let mut i = 0;
     let n = scan_ch(data, b'<');
     if n == 0 { return None; }
@@ -561,7 +627,7 @@ pub fn is_escaped(data: &str, loc: usize) -> bool {
 }
 
 // Remove backslash escapes and resolve entities
-pub fn unescape<'a>(input: &'a str) -> Cow<'a, str> {
+pub fn unescape(input: &str) -> Cow<str> {
     if input.find(|c| c == '\\' || c == '&' || c == '\r').is_none() {
         Borrowed(input)
     } else {
